package sun.security.util;

import java.security.AccessController;
import java.util.Enumeration;
import java.util.HashSet;
import java.io.FileInputStream;
import java.security.KeyStore;
import java.io.File;
import java.security.PrivilegedAction;
import java.util.Collections;
import sun.security.x509.X509CertImpl;
import java.security.cert.X509Certificate;
import java.util.Set;

public class AnchorCertificates
{
    private static final Debug debug;
    private static final String HASH = "SHA-256";
    private static Set<String> certs;
    
    public static boolean contains(X509Certificate x509Certificate) {
        int contains = AnchorCertificates.certs.contains(X509CertImpl.getFingerprint("SHA-256", x509Certificate));
        if (contains && AnchorCertificates.debug != null) {
            AnchorCertificates.debug.println("AnchorCertificate.contains: matched " + x509Certificate.getSubjectDN());
        }
        return contains;
    }
    
    private AnchorCertificates() {
    }
    
    static {
        debug = Debug.getInstance("certpath");
        AnchorCertificates.certs = Collections.emptySet();
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                File file = new File(System.getProperty("java.home"), "lib/security/cacerts");
                try {
                    KeyStore instance = KeyStore.getInstance("JKS");
                    try (final FileInputStream stream = new FileInputStream(file)) {
                        instance.load(stream, (char[])null);
                        AnchorCertificates.certs = new HashSet();
                        Enumeration<String> aliases = instance.aliases();
                        while (aliases.hasMoreElements()) {
                            String alias = (String)aliases.nextElement();
                            if (!alias.contains(" [jdk")) {
                                continue;
                            }
                            AnchorCertificates.certs.add(X509CertImpl.getFingerprint("SHA-256", (X509Certificate)instance.getCertificate(alias)));
                        }
                    }
                }
                catch (Exception ex) {
                    if (AnchorCertificates.debug != null) {
                        AnchorCertificates.debug.println("Error parsing cacerts");
                        ex.printStackTrace();
                    }
                }
                return null;
            }
        });
    }
}
